pkgHome		= "http://menumaker.sourceforge.net"
pkgName		= "MenuMaker"
pkgVer		= "0.99.2"




author		= "Oleg A. Khlybov"
email		= "fougas@mail.ru"
copyright	= "(c) 2002-2005"




verbose			= False
omitEmptyMenu	= True  # Whether to omit or to retain empty menu
writeFullMenu	= True  # Whether to generate the entire menu or just a part to be included into the hand crafted one




import sys




def msg(s, newline = True) :
	if verbose :
		sys.stderr.write(s)
		if newline :
			sys.stderr.write("\n")




from Prophet import warn, fatal




if not (sys.version_info[0] >= 2 and sys.version_info[1] >= 3) :
    fatal("Python 2.3 or higher is required")




def indent(level) :
	"""Return indentation string for specified level"""
	x = ""
	for i in range(0, level) : x += "	" # FIXME : make this stuff faster
	return x




def setFrontend(module) :
	"""Specify the frontend to use"""
	for x in (App, Menu, Root) :
		x.__bases__ = (getattr(module, x.__name__),) + x.__bases__




class Entry(object) :
	"""Basic menu class"""
	# Alignment specifiers
	MenuFloat	= 0 # Element is subject to sorting
	MenuTop		= 1 # Element should be placed at the menu top retaining the original order
	MenuBottom	= 2 # ...at the bottom...

	def __init__(self, *args) :
		super(Entry, self).__init__(*args)
		self.alignment = Entry.MenuFloat




class Menu(Entry, list) :
	"""Class which may contain other (sub)menus and single entries"""

	def distribute(self, apps) :
		"""Distribute the application entries among the menu's submenus and then try to place the
			rest into itself. Return the list of the apps that didn't fit anywhere"""
		for x in self :
			if isinstance(x, Menu) :
				apps = x.distribute(apps)
		rest = []
		for x in apps :
			if self.suitable(x) :
				self.append(App(x))
			else :
				rest.append(x)
		return rest

	def empty(self) :
		"""Check whether the menu is empty, i.e. doesn't contain any meaningful entries
			(non-empty submenus, for ex.)"""
		for x in self :
			if isinstance(x, Menu) :
				if not x.empty :
					return False
			else :
				return False
			# TODO : separators do not count
		return True
	empty = property(empty)

	def emit(self, *args) :
		if omitEmptyMenu and self.empty :
			return []
		else :
			return super(Menu, self).emit(*args) # Delegating it in hope that there is frontend-specific emit() down the inheritance graph

	def suitable(self, app) :
		"""Simple suitability test -- check whether the app falls into any of specified categories"""
		return len(self.keywords & app.keywords) > 0
		# sets.Set has a (broken???) implementation of the intersection operation (&)
		# therefore one has to use built-in set type which in turn imposes a restriction
		# on minimum allowed version of Python (2.4+)

	def arrange(self) :
		top = []
		float = []
		bottom = []
		for x in self :
			if isinstance(x, Menu) :
				x.arrange()
			if   x.alignment == Entry.MenuFloat :
				float.append(x)
			elif x.alignment == Entry.MenuTop :
				top.append(x)
			elif x.alignment == Entry.MenuBottom :
				bottom.append(x)
		float.sort( lambda x, y : cmp(x.name.lower(), y.name.lower()) )
		self[:] = top + float + bottom




class App(Entry) :
	"""Application entry"""

	def __init__(self, app) :
		super(App, self).__init__()
		self.name = app.name
		self.app = app




import fnmatch




from Prophet.Categories import *
from Keywords import Set as KwS, Keyword as Kw




# KDE-specific keywords
#XKM	= Kw("X-KDE-More")
#XKI	= Kw("X-KDE-information")
#XKSLF	= Kw("X-KDE-settings-looknfeel")
#XKSS	= Kw("X-KDE-settings-sound")
#XKSY	= Kw("X-KDE-settings-system")
#XKSA	= Kw("X-KDE-settings-accessibility")
#XKSC	= Kw("X-KDE-settings-components")
#XKSN	= Kw("X-KDE-settings-network")
#XKSX	= Kw("X-KDE-settings-security")
#XKSW	= Kw("X-KDE-settings-webbrowsing")
#XKSD	= Kw("X-KDE-settings-desktop")
#XKSP	= Kw("X-KDE-settings-peripherials")
#XKSH	= Kw("X-KDE-settings-hardware")
#XKSW	= Kw("X-KDE-settings-power")
#




class Root(Menu) :
	name = "Applications"

	def __init__(self) :
		subs = (
			DevApps, Editors, OfficeApps, ScienceApps, NetApps, GraphicApps,
			Multimedia, Edutainment, Games, Shells, SystemApps, Utils, Misc
		)
		super(Root, self).__init__([x() for x in subs])




class DevApps(Menu) :
	name = "Development"
	keywords = KwS(Development, Building, Debugger, IDE, GUIDesigner, Profiling, RevisionControl, Translation, WebDevelopment)




class Editors(Menu) :
	name = "Editors"
	keywords = KwS(TextEditor)




class Misc(Menu) :
	"""Menu that contains the entries that didn't fit elsewhere"""
	name = "Other"

	def suitable(self, app) :
		return True




class Shells(Menu) :
	name = "Shells"
	keywords = KwS(Shell, TerminalEmulator, FileManager, Emulator)




class Utils(Menu) :
	name = "Utilities"
	keywords = KwS(Utility, Accessibility, Calculator, Clock, Applet, Archiving, TrayIcon)




class KDESettings(Menu) :
	name = "KDE"

	def suitable(self, app) :
		for x in app.keywords :
			if fnmatch.fnmatchcase(x, "X-KDE-settings*") :
				return True
		return False




class SystemSettings(Menu) :
	name = "Settings"
	keywords = KwS(Settings, DesktopSettings, HardwareSettings, Screensaver)

	def __init__(self) :
		subs = (KDESettings,)
		super(SystemSettings, self).__init__([x() for x in subs])




class SystemApps(Menu) :
	name = "System"
	keywords = KwS(System, Filesystem, Monitor, Security, PackageManager)

	def __init__(self) :
		subs = (SystemSettings,)
		super(SystemApps, self).__init__([x() for x in subs])




class ScienceApps(Menu) :
	name = "Science"
	keywords = KwS(Electronics, Engineering, Science, Astronomy, Biology, Chemistry, Geology, Math, MedicalSoftware, Physics)




class Edutainment(Menu) :
	name = "Education"
	keywords = KwS(Education, Art, Construction, Music, Languages, Teaching)




class AdventureGames(Menu) :
	name = "Adventure"
	keywords = KwS(AdventureGame, RolePlaying)




class ArcadeGames(Menu) :
	name = "Arcade"
	keywords = KwS(ArcadeGame, ActionGame)




class BoardGames(Menu) :
	name = "Board"
	keywords = KwS(BoardGame)




class CardGames(Menu) :
	name = "Card"
	keywords = KwS(CardGame)




class PuzzleGames(Menu) :
	name = "Puzzle"
	keywords = KwS(LogicGame, BlocksGame)




class SimGames(Menu) :
	name = "Simulation"
	keywords = KwS(Simulation)




class SportsGames(Menu) :
	name = "Sports"
	keywords = KwS(SportsGame)




class StrategyGames(Menu) :
	name = "Strategy"
	keywords = KwS(StrategyGame)




class ToyGames(Menu) :
	name = "Amusement"
	keywords = KwS(Amusement, KidsGame)




class Games(Menu) :
	name = "Games"
	keywords = KwS(Amusement, Game, ActionGame, AdventureGame, ArcadeGame, BoardGame, BlocksGame, CardGame, KidsGame, LogicGame, RolePlaying, Simulation, SportsGame, StrategyGame)

	def __init__(self) :
		subs = (
			AdventureGames, ArcadeGames, BoardGames, CardGames, 
			PuzzleGames, SimGames, SportsGames, StrategyGames, ToyGames
		)
		super(Games, self).__init__([x() for x in subs])




class Multimedia(Menu) :
	name = "Multimedia"
	keywords = KwS(AudioVideo, Audio, Midi, Mixer, Sequencer, Tuner, Video, TV, AudioVideoEditing, Player, Recorder, DiscBurning)




class NetApps(Menu) :
	name = "Network"
	keywords = KwS(Network, Dialup, InstantMessaging, IRCClient, FileTransfer, HamRadio, News, Email, P2P, RemoteAccess, Telephony, WebBrowser)




class GraphicApps(Menu) :
	name = "Graphics"
	keywords = KwS(Graphics, x2DGraphics, x3DGraphics, VectorGraphics, RasterGraphics, Scanning, OCR, Photograph, Viewer)




class OfficeApps(Menu) :
	name = "Office"
	keywords = KwS(Office, Calendar, ContactManagement, Database, Dictionary, Chart, Finance, FlowChart, PDA, ProjectManagement, Presentation, Spreadsheet, WordProcessor)
